from abc import ABC, abstractmethod

# LLM对象
class ModelRecord:
    code: str
    api_base: str
    api_key: str
    model_name: str
    api_style: str
    config:str

    def __init__(self, api_base, api_key, code, model_name, api_style, config):
        self.api_base = api_base
        self.api_key = api_key
        self.code = code
        self.model_name = model_name
        self.api_style = api_style
        self.config = config

# 提示词对象
class PromptRecord:
    code: str
    system_prompt: str
    user_prompt:str 
    temperature: float
    model: ModelRecord
    api_style: str
    id: int

    def __init__(self, code, system_prompt, user_prompt, temperature, model, id):
        self.system_prompt = system_prompt
        self.user_prompt = user_prompt
        self.temperature = temperature
        self.model = model
        self.code = code
        self.id = id

# 聊天记录对象
class ChatRecord:
    trace_id: str
    prompt:str  
    completion:str 
    model:str 
    prompt_tokens:int
    completion_tokens:int
    response_s:float
    cached_tokens:int

    auto_id:int
    created_at:str

    def __init__(self, trace_id, prompt, completion, model, prompt_tokens, completion_tokens, response_s, cached_tokens):
        self.trace_id = trace_id
        self.prompt = prompt
        self.completion = completion
        self.model = model
        self.prompt_tokens = prompt_tokens
        self.completion_tokens = completion_tokens
        self.response_s = response_s
        self.cached_tokens = cached_tokens

class BaseStore(ABC):
    api_host: str
    api_key: str
    db_code: str
    model_table_code: str
    prompt_table_code: str
    chat_log_table_code: str
    
    @abstractmethod
    def get_model(self, code: str) -> ModelRecord:
        """_summary_

        Args:
            code (str): _description_

        Returns:
            PromptRecord: _description_
        """
        pass

    @abstractmethod
    def get_prompt(self, code: str) -> PromptRecord:
        """获取提示词模板配置
        
        Args:
            code: 模板名称
            
        Returns:
            dict: 模板配置信息，包含 prompt 等
        """
        pass

    @abstractmethod
    def save_chatlog(self, fields:dict):
        """保存聊天记录
        
        Args:
            chat_data: 聊天记录数据
            
        Returns:
            bool: 保存是否成功
        """
        pass


from openai import OpenAI
from openai.types.chat import ChatCompletion
from llm_store.store.basestore import BaseStore, PromptRecord, ModelRecord
from llm_store.store.baserowstore import BaseRowStore
import time
from jinja2 import Template, Environment


class LLMGateway:
    store: BaseStore

    def __init__(self, store: BaseStore):
        self.store = store

    def _get_llm_client(self, model: ModelRecord):
        """获取LLM客户端
        """
        if model.api_style == "openai":
            return OpenAI(
                base_url=model.api_base,
                api_key=model.api_key
            )
        else:
            raise ValueError(f"Unsupported API style: {model}")

    def get_model(self, model_code: str) -> ModelRecord:
        return self.store.get_model(model_code)

    def get_client(self, model: ModelRecord) -> OpenAI:
        return self._get_llm_client(model)

    def completions(self, model: ModelRecord, messages, is_save_log=True, trace_id=None, prompt_record_id=None, **kwargs):
        llm_client = self._get_llm_client(model)
        start_time = time.time()
        chatCompletion = llm_client.chat.completions.create(messages=messages, model=model.model_name,
                                                            **kwargs)
        end_time = time.time()
        response_cost = round(end_time - start_time, 2)

        if is_save_log:
            log_record =self.save_log(
                messages= messages,
                model =model,
                chatCompletion= chatCompletion,
                time_cost = response_cost,
                traceid = trace_id,
                prompt_record_id = prompt_record_id
            )
            print("save baserow chat log:"+  str(log_record.id))

        return chatCompletion

    def render_prompt(self, prompt_code: str, params={}) -> tuple[ModelRecord, PromptRecord, list[dict]]:
        # 获取 Prompt 记录
        prompt = self.store.get_prompt(prompt_code)

        # 渲染参数
        env = Environment(
            trim_blocks=True,  # 去掉行尾的空白字符
            lstrip_blocks=False  # 去掉行首的空白字符
        )

        system_template = env.from_string(prompt.system_prompt)
        render_system_prompt = system_template.render(params)

        messages = [{"role": "system", "content": render_system_prompt}]
        if prompt.user_prompt:
            user_template = env.from_string(prompt.user_prompt)
            render_user_prompt = user_template.render(params)

            messages.append({"role": "user", "content": render_user_prompt})

        return prompt.model, prompt, messages

    def save_log(self, messages, model: ModelRecord,chatCompletion: ChatCompletion,time_cost:float, traceid: str, prompt_record_id: int = None):
        content_str = "----".join(message['content'] for message in messages)

        cached_token = None
        if 'prompt_tokens_details' in chatCompletion.usage:
            cached_token = chatCompletion.usage.prompt_tokens_details.cached_tokens

        fields = {
            "prompt": content_str,
            "completion": chatCompletion.choices[0].message.content,
            "model": model.model_name,
            "prompt_tokens": chatCompletion.usage.prompt_tokens,
            "completion_tokens": chatCompletion.usage.completion_tokens,
            "response_s": time_cost,
            "cached_tokens": cached_token,
            "trace_id": traceid
        }

        if prompt_record_id:
            fields['prompt_record'] = [prompt_record_id]

        return self.store.save_chatlog(fields)


if __name__ == "__main__":
    store = BaseRowStore(
        api_host="https://aiadmin.yucekj.com",
        api_key="Zfsk1r7M7FyMw9qRjAZcPxkTLVy3x8U7",
        db_code="3",
        model_table_code="614",
        prompt_table_code="308",
        chat_log_table_code="309")
    llm_gateway = LLMGateway(store)

    # 标准使用模式
    # -- 渲染 Prompt
    # prompt_code = "test_template"
    # model_record, prompt_record, messages = llm_gateway.render_prompt(prompt_code, {"topic": "科幻"})
    #
    # # -- 对话
    # chatCompletion, time_cost = llm_gateway.completions(model_record, messages)
    # print(chatCompletion, time_cost)
    #
    # # -- 保存对话记录
    # trace_id = "1234567890"
    # log = llm_gateway.save_log(trace_id, model_record, prompt_record.id)
    # print(log)

    # 不使用远程 Prompt 模式
    messages = [{"role": "system", "content": "今天星期几"}]

    # -- 获取 模型信息
    model_code = "litellm-us"
    model_record = llm_gateway.get_model(model_code)

    # -- 执行对话
    chatCompletion, time_cost = llm_gateway.completions(model_record,messages)

    # -- 保存日志
    trace_id = "1234567890"
    log = llm_gateway.save_log(messages, model_record, traceid=trace_id)
    print(log)





from llm_store.store.basestore import BaseStore, ModelRecord,PromptRecord
from baserowsdk.client import Client
class BaseRowStore(BaseStore):
    client: Client

    def __init__(self, api_host, api_key,db_code, model_table_code, prompt_table_code, chat_log_table_code):
        self.api_host = api_host
        self.api_key = api_key
        self.model_table_code = model_table_code
        self.prompt_table_code = prompt_table_code
        self.chat_log_table_code = chat_log_table_code
        self.db_code = db_code

        self.client = Client(api_key=api_key, base_url=api_host)

    def get_model(self, code: str) -> ModelRecord:
        filters = {"filter_type":"AND","filters":[{"type":"equal","field":"code","value":code}],"groups":[]}
        model_data = self.client.base(int(self.db_code)).table(self.model_table_code).rows(filters=filters)
        assert len(model_data) == 1 
        model_data = model_data[0]
        
        rtn = ModelRecord(
                api_base=model_data.api_base,
                api_key=model_data.api_key,
                code=model_data.code,
                model_name=model_data.model_name,
                api_style=model_data.api_style,
                config=model_data.config)
        
        # 下拉框处理
        rtn.api_style = rtn.api_style['value']
        return rtn
   
   
    def get_prompt(self, code: str) -> PromptRecord:
        filters = {"filter_type":"AND","filters":[{"type":"equal","field":"code","value":code}],"groups":[]}
        model_data = self.client.base(self.db_code).table(self.prompt_table_code).rows(filters=filters)
        assert len(model_data) == 1 
        row = model_data[0]
        
        model = ModelRecord(
                api_base=row.api_base[0]['value'],
                api_key=row.api_key[0]['value'],
                code=row.code,
                model_name=row.model_name[0]['value'],
                api_style=row.api_style[0]['value']['value'],
                config=row.config[0]['value'] if row.config else None)
        

        rtn = PromptRecord(
            code =row.code,
            system_prompt=row.system_prompt,
            user_prompt=row.user_prompt,
            temperature=row.temperature,
            model = model,
            id = row.id
        )
        rtn.api_style = model.api_style
        return rtn
     
    
    def save_chatlog(self, fields:dict):
        return self.client.base(int(self.db_code)).table(self.chat_log_table_code).create(fields)
